#!/bin/bash
source /itmonitor/thresholds.conf &>/dev/null

awk -v chkCPU=$chkCPU -v chkDISK=$chkDISK -v chkMEM=$chkMEM -v CPUT=$CPUT -v DISKT=$DISKT -v MEMT=$MEMT -v SERVNAME="$SERVNAME" -v SERVOS=$SERVOS -v SERVCHANNEL=$SERVCHANNEL  'BEGIN {
#Generate the Header for the Payload including servername, os and channel to report in Slack
printf "{\n"
printf " \"servername\" : %s",SERVNAME
printf " \"os\" : %s",SERVOS
printf " \"channel\" : %s",SERVCHANNEL

#Block that checks Disk Space and generate this segment of Payload
#it uses df command and ignores /boot, it compares the used percentaje for each mount point
#against the threshold and generates a line to send the alert
diskband = 0
if ( chkDISK == 1 ) {
	while("df -hP | grep -v /boot" | getline) {
		if ( $NF ~ "/" ) {
			hdd= $5*1
			#check if alert is triggered
			if(hdd>(DISKT*100)) {
				if (diskband == 0 ) {
					diskband = 1
					printf "\n\"payload\": \"*Alerts:*\\n*=> Disk:*\\n"
				}
				#alert was triggered so print a message with full info about that mount point
				printf "%s %s Usage: %s/%s (%s disk space used)\\n", $1,$6,$3,$2,$5
			}
		}
	}
}

#Block that checks CPU based in the load avg for the last 5 min and generate this segment of Payload
#to be acurate in the % it takes the number of logical procesors counting the core id from /proc/cpuinfo
#if the alert is triggered it runs ps to get the top 5 proccess in use of cpu and send them with the alert
procband = 0
if ( chkCPU == 1 ) {
	while( getline  < "/proc/loadavg" ) {
		cpu=$2
		cmd="cat /proc/cpuinfo | grep \"core id\" | wc -l"
		while( cmd | getline) {
			cores=$1
		}
		#check if alert is triggered
		if(cpu/cores > CPUT) {
			if (procband == 0 ){
				procband = 1
				if (diskband == 1){
					printf "*=> CPU:*\\n"
				}
				else{
					printf "\n\"payload\": \"*Alerts:*\\n*=> CPU:*\\n"
				}
			}
			printf "CPU Load: %.2f%\\n", $1/cores*100
			#The alert was triggered so attach the top 5 CPU consuming proccess
			while ("ps -Ao user,uid,comm,pid,pcpu --sort=-pcpu | head -n 6" | getline) {
				printf "%s\\n",$0
			}
		}
	}
}


#Block that checks MEMORY based in the sysstat to get the avg for the last for days and generate this segment of payload
#first it takes the avg for each day and generate a new avg based in the last 4 days then it get the % of used memory including cached memory
#and compares it taking cosideration for the % of increase to generate the alert
memband = 0
if(chkMEM == 1) {
	memprom=0
	memcount=0
	#based in the last 4 days generates the avg
	while("find /var/log/sysstat/ -mtime -4 | grep -P 'sa[0-9]'" | getline) {
		cmd="sar -r -f " $0 " | grep Ave"
		while(cmd | getline){
			memcount=memcount+1
			memprom=memprom+$4
		}
	}
	memest=memprom/memcount
	#calculates the % of memory actually in use considering the cached (buffered) memory
	while( "free -m"| getline) {
		if( $1 ~ "Mem") {
			memt=$2
		}
		if( $1 ~ "-/+") {
			memf=$4
		}
	}
	mem=(memt-memf)*100/memt
	#check if the alert is triggered
	if(mem > (memest+(memest*MEMT))) {
		if ( memband == 0 ) {
			memband=1
			if (diskband == 1 || procband == 1) {
				printf "*=> Memory:*\\n"
			}
			else {
				printf "\n\"payload\": \"*Alerts:*\\n*=> Memory:*\\n"
			}
		}
		#puts a message with info concerning the metric
		printf "Memory Usage: %s/%sMB (%.2f%)\\n", $3,$2,$3*100/$2  
	}
}

#this is just a sanity check to finish the payload correctly
if(memband == 1 || procband == 1 || diskband == 1) {
	printf "\""
}
printf "\n}\n"
}' >| /itmonitor/payload.json


awk 'BEGIN {
#check for error about non existing config file
if ( system ("[ -f /itmonitor/error.lck ]") == 0 ) {
	system("rm -f /itmonitor/error.lck")
}
else {
	#as the payload has to be in 1 line to ensure Zapier reads it
	#we know the number of lines so we check if payload has an alert or not
	while("wc -l /itmonitor/payload.json" | getline) {
		if($1 > 6) {
			printf "sending payload\n"
			#this generates the command to send the payload based in send_payload.sh script
			cmd="/itmonitor/send_payload.sh"
			system(cmd)
			printf "payload sent\n"
		}
	}
	
}
}'
